@@ -15,7 +15,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD %iptcCharset);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.53';
+$VERSION = '1.54';
 
 %iptcCharset = (
     "\x1b%G"  => 'UTF8',
@@ -1073,11 +1073,18 @@ sub ProcessIPTC($$$)
                 }
                 $et->FoundTag('CurrentIPTCDigest', $md5);
             }
-        } elsif ($Image::ExifTool::MWG::strict and $$et{FILE_TYPE} =~ /^(JPEG|TIFF|PSD)$/) {
-            # ignore non-standard IPTC while in strict MWG compatibility mode
-            $et->Warn("Ignored non-standard IPTC at $path");
-            return 1;
         } else {
+            if (($Image::ExifTool::MWG::strict or $et->Options('Validate')) and 
+                $$et{FILE_TYPE} =~ /^(JPEG|TIFF|PSD)$/)
+            {
+                if ($Image::ExifTool::MWG::strict) {
+                    # ignore non-standard IPTC while in strict MWG compatibility mode
+                    $et->Warn("Ignored non-standard IPTC at $path");
+                    return 1;
+                } else {
+                    $et->Warn("Non-standard IPTC at $path", 1);
+                }
+            }
             # extract non-standard IPTC
             my $count = ($$et{DIR_COUNT}{IPTC} || 0) + 1;  # count non-standard IPTC
             $$et{DIR_COUNT}{IPTC} = $count;
