@@ -21,7 +21,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::XMP;
 use Image::ExifTool::GPS;
 
-$VERSION = '1.06';
+$VERSION = '1.07';
 
 sub ExtractObject($$;$);
 sub Get24u($$);
@@ -318,7 +318,7 @@ sub ExtractObject($$;$)
 #------------------------------------------------------------------------------
 # Process binary PLIST data (ref 2)
 # Inputs: 0) ExifTool object ref, 1) DirInfo ref, 2) tag table ref
-# Returns: 1 on success
+# Returns: 1 on success (and returns plist value as $$dirInfo{Value})
 sub ProcessBinaryPLIST($$$)
 {
     my ($et, $dirInfo, $tagTablePtr) = @_;
@@ -328,8 +328,15 @@ sub ProcessBinaryPLIST($$$)
     SetByteOrder('MM');
 
     unless ($$dirInfo{RAF}) {
-        my $buf2 = substr(${$$dirInfo{DataPt}}, $$dirInfo{DirStart} || 0, $$dirInfo{DirLen});
-        $$dirInfo{RAF} = new File::RandomAccess(\$buf2);
+        my $dataPt = $$dirInfo{DataPt};
+        my $start = $$dirInfo{DirStart};
+        if ($start or ($$dirInfo{DirLen} and $$dirInfo{DirLen} != length $$dataPt)) {
+            my $buf2 = substr($$dataPt, $start || 0, $$dirInfo{DirLen});
+            $$dirInfo{RAF} = new File::RandomAccess(\$buf2);
+        } else {
+            $$dirInfo{RAF} = new File::RandomAccess($dataPt);
+        }
+        my $strt = $$dirInfo{DirStart} || 0;
     }
     # read and parse the trailer
     my $raf = $$dirInfo{RAF};
@@ -359,8 +366,8 @@ sub ProcessBinaryPLIST($$$)
     );
     # position file pointer at the top object, and extract it
     $raf->Seek($table[$topObj], 0) or return 0;
-    my $result = ExtractObject($et, \%plistInfo);
-    return defined $result ? 1 : 0;
+    $$dirInfo{Value} = ExtractObject($et, \%plistInfo);
+    return defined $$dirInfo{Value} ? 1 : 0;
 }
 
 #------------------------------------------------------------------------------
