@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.40';
+$VERSION = '1.41';
 
 sub WritePS($$);
 sub ProcessPS($$;$);
@@ -328,7 +328,10 @@ sub ProcessPS($$;$)
     my ($data, $dos, $endDoc, $fontTable, $comment);
 
     # allow read from data
-    $raf = new File::RandomAccess($$dirInfo{DataPt}) unless $raf;
+    unless ($raf) {
+        $raf = new File::RandomAccess($$dirInfo{DataPt});
+        $et->VerboseDir('PostScript');
+    }
 #
 # determine if this is a postscript file
 #
@@ -671,7 +674,7 @@ This code reads meta information from EPS (Encapsulated PostScript), PS
 
 =head1 AUTHOR
 
-Copyright 2003-2015, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
